import itertools
import json
import os
import random
import re
from typing import Any
from json_dataset_builder import extract_sophora_text, clean_html

"""
Description:
    This script merges the contents of JSON files from a specified directory into new JSON files.
    Unlike sequential merging, it uses a combinatorial approach:
    it generates all possible combinations of the source files based on a given group size (step).
    Each combination is saved as a separate JSON file.

Core Idea:
    - Let n = total number of source JSON files.
    - Let k = group size (step).
    - The number of possible combinations is calculated using the binomial coefficient formula:
        C(n, k) = n! / (k! * (n - k)!)
    - For example, if n = 75 and k = 10, the number of combinations will be extremely large,
      so resource limitations should be considered.

Warning:
    - The number of combinations grows exponentially with step.
    - For large n and k, consider limiting the number of combinations or using random sampling.
"""


import itertools
import random

def merge2(step: int = 2, limit: int = 200):
    json_paths = load_json_files()
    combos = itertools.combinations(json_paths, step)

    # Reservoir sampling
    reservoir = []
    for i, combo in enumerate(combos, start=1):
        if len(reservoir) < limit:
            reservoir.append(combo)
        else:
            j = random.randint(1, i)
            if j <= limit:
                reservoir[j - 1] = combo

    # Теперь reservoir содержит случайные 200 комбинаций
    for idx, combo in enumerate(reservoir):
        merged = {"source_files": [], "file_contents": []}
        for path in combo:
            json_i = load_json_file(path)
            if has_defect(json_i):
                continue
            merged["source_files"].append(get_filename(path))
            merged["file_contents"].append(parse_json(json_i))

        write_to_file(idx, merged)

def get_filename(path: str) -> str:
    return re.search(r'[^/]+$', path).group(0)

def load_json_files():
    input_folder = "/home/nurseiit/PycharmProjects/wdr-article-semantic-chunking2/Artikel_WDR_NRW"

    files = sorted([f for f in os.listdir(input_folder) if f.endswith('.json')])
    res_files = [os.path.join(input_folder, f) for f in files]
    if len(res_files) < 2:
        raise Exception("Not enough JSON files to process.")
    return res_files



def load_json_file(path):
    with open(path, 'r', encoding='utf-8') as f:
        return json.load(f)



def create_file_name(json_paths: list[str], t: int) -> str:
    return re.search(r'[^/]+$', json_paths[t]).group(0)


def write_to_file(i: int, merged: dict[str, list[Any]]):
    output_filename = f'merged_filtered_{i}.json'
    from pathlib import Path
    folder_name = "data"
    file_name = f"{output_filename}"
    path = Path.cwd() / folder_name / file_name
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, 'w', encoding='utf-8') as outfile:
        json.dump(merged, outfile, ensure_ascii=False, indent=4)


def parse_json(json):
    res_json = {}
    res_json["sophora:tags"] = clean_html(json.get("sophora:tags"))
    res_json["wdr:leadin"] = clean_html(json.get("wdr:leadin"))
    res_json["wdr:schlagzeile"] = clean_html(json.get("wdr:schlagzeile"))
    res_json["wdr:teasertext"] = clean_html(json.get("wdr:teasertext"))
    res_json["sophora-extension:text"] = clean_html(extract_sophora_text(json.get("wdr:text", [])))
    return res_json


def has_defect(json_file) -> bool:
    sophora_text = extract_sophora_text(json_file.get("wdr:text", []))
    if len(sophora_text) < 4:
        return True
    else:
        return False


if __name__ == "__main__":
    merge2(4,200)



# Define the file name
import json
import os
from typing import List, Dict, Tuple
from pathlib import Path

import pandas as pd

file_name = "output.txt"
folder_name = "content"

# Define the text content


def write_to_file(text: str, output_filename=f'merged_filtered_.txt', line_number=None):
    from pathlib import Path

    file_name = f"{output_filename}"
    path = Path.cwd() / folder_name / file_name
    path.parent.mkdir(parents=True, exist_ok=True)

    words = text.split()
    if line_number:
        with open(path, "a", encoding="utf-8") as f:
            f.write(f"{line_number}  ")
    with open(path, "a", encoding="utf-8") as f:
        for i in range(0, len(words), 13):
            line = " ".join(words[i:i + 13])
            f.write(line + "\n")
        f.write("\n")


def write_split(output_filename=f'merged_filtered_.txt', should_write=True):
    if should_write:
        from pathlib import Path
        file_name = f"{output_filename}"
        path = Path.cwd() / folder_name / file_name
        path.parent.mkdir(parents=True, exist_ok=True)

        with open(path, "a", encoding="utf-8") as f:
            f.write("******************************************\n")


def write_to_file_with_index(text: str, output_filename=f'merged_filtered_.txt', line_number=None, should_write=True):
    if should_write:
        from pathlib import Path
        file_name = f"{output_filename}"
        path = Path.cwd() / folder_name / file_name
        path.parent.mkdir(parents=True, exist_ok=True)

        words = text.split()
        if line_number:
            with open(path, "a", encoding="utf-8") as f:
                f.write(f"{line_number}  ")
        with open(path, "a", encoding="utf-8") as f:
            for i in range(0, len(words), 13):
                line = " ".join(words[i:i + 13])
                f.write(line + "\n")
            f.write("\n")


def clean(output_filename=f'merged_filtered_.txt'):
    from pathlib import Path
    file_name = f"{output_filename}"
    path = Path.cwd() / folder_name / file_name
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "w", encoding="utf-8"):
        pass


import csv



FILE = "File"
MATCHES = "matches"
EXPECTED_BOUNDARY = "boundary"
SIMILARITIES = "similarities"
POSSIBLE_BREAKS = "possible_breaks"
SOURCE_COUNT = "source_count"
MATCH_PERCENTAGE = "percentage_match"


def save_pair_to_csv(key: str, possible_breaks: List[int],# todo rename
                     expected_boundary: list[int], source_count: int,
                     file_path:str):
    fieldnames = [FILE, EXPECTED_BOUNDARY, POSSIBLE_BREAKS, SOURCE_COUNT,MATCHES, MATCH_PERCENTAGE]

    p = Path(file_path)
    if p.parent:  # only if there's a directory component
        p.parent.mkdir(parents=True, exist_ok=True)

    # os.makedirs(os.path.dirname(file_path), exist_ok=True)

    with open(file_path, "a", newline="", encoding="utf-8") as f:
        writer = csv.DictWriter(f, fieldnames=fieldnames)
        if f.tell() == 0:  # if file is empty, write header
            writer.writeheader()
        matches = match_with_tolerance2(expected_boundary, possible_breaks, tolerance=3)
        writer.writerow({
            FILE: key,
            POSSIBLE_BREAKS: possible_breaks,
            EXPECTED_BOUNDARY: expected_boundary,
            SOURCE_COUNT: source_count,
            MATCHES: matches,
            MATCH_PERCENTAGE: find_percent_in_dict(matches)
        })

def save_result_tocsv(combination:str,result:float):
    file_name = "result.csv"
    p = Path(file_name)
    if p.parent:
        p.parent.mkdir(parents=True, exist_ok=True)

    with open(file_name, "a", newline="", encoding="utf-8") as f:
        writer = csv.DictWriter(f, fieldnames=["combination", "result"])
        if f.tell() == 0:
            writer.writeheader()
        writer.writerow({
            "combination": combination,
            "result": result
        })


def find_percent_in_dict(d: Dict[int, bool]):
    total_keys = len(d)
    if total_keys == 0:
        return 0
    true_count = sum(1 for v in d.values() if v)
    percent = (true_count / total_keys) * 100
    return round(percent)



def match_with_tolerance(expected_boundary: list[int],#todo test
                         possible_breaks: list[int],
                         tolerance: int = 3) -> dict[int, bool]:
    """
    Returns a dict mapping expected index -> True/False depending on whether there
    exists a predicted value within ±tolerance from the expected value.
    """
    matches: dict[int, bool] = {}
    for idx, exp in enumerate(possible_breaks):
        # True if any predicted is within ±tolerance
        is_match = any(abs(exp - p) <= tolerance for p in expected_boundary[:-1])
        matches[idx] = is_match
    if len(matches)<len(expected_boundary[:-1]):
        for i in range(len(matches)-1,len(expected_boundary)-len(matches)):
            matches.update({i+1: False})
    return matches

def match_with_tolerance2(expected_boundary: list[int],#todo test
                         possible_breaks: list[int],
                         tolerance: int = 3) -> dict[int, bool]:
    """
    Returns a dict mapping expected index -> True/False depending on whether there
    exists a predicted value within ±tolerance from the expected value.
    """
    matches = {}
    for index, expected in enumerate(expected_boundary[:-1]):
        matches[index] = False
        for found in possible_breaks:
            if abs(expected - found) <= tolerance:
                matches[index] = True
    return matches


# v = match_with_tolerance(
#     expected_boundary=[12, 25, 38, 50],
#     possible_breaks=[26, 35], )
#
# print(v)



import numpy as np
import pandas as pd
from sentence_transformers import SentenceTransformer
from sklearn.metrics.pairwise import cosine_similarity

from wdr_article_semantic_chunking_2.computer.plotter import plot_sliding_window
from wdr_article_semantic_chunking_2.text_util.sw_writer import save_pair_to_csv, MATCH_PERCENTAGE, save_result_tocsv
from wdr_article_semantic_chunking_2.text_util.text_extractor import extract_texts_and_write_to_file
from wdr_article_semantic_chunking_2.util.util import get_path_for_plotting, get_path_for_csv


def compute(
        window_size,
        min_gap,
        model_name):
    model = SentenceTransformer(model_name)
    combination_name = f"model_{model_name}_w_{window_size}_m_{min_gap}"
    for i in range(0, 100):
        file_name = f"merged_filtered_{i}.json"
        blocks, expected_boundary, source_count, _ = extract_texts_and_write_to_file(file_name, False)

        boundaries, scores, indices = segment_topics_window(blocks, window_size=window_size, min_gap=min_gap,
                                                            model=model)

        plot_sliding_window(
            indices=indices,
            scores=scores,
            expected_boundaries=expected_boundary,
            file_name=get_path_for_plotting(combination_name, file_name),
            window_size=window_size
        )

        save_pair_to_csv(
            key=file_name,
            possible_breaks=boundaries,
            expected_boundary=expected_boundary,
            source_count=source_count,
            file_path=get_path_for_csv(combination_name)
        )
    df = pd.read_csv(get_path_for_csv(combination_name), usecols=[MATCH_PERCENTAGE])
    save_result_tocsv(combination_name, df.mean().iloc[0])


def segment_topics_window(
        blocks,
        window_size,
        min_gap,
        model
):
    embeddings = model.encode(blocks)

    scores = []
    indices = []

    for i in range(window_size, len(blocks) - window_size):
        left = embeddings[i - window_size:i]
        right = embeddings[i:i + window_size]

        left_mean = optimize_embddings(left)
        right_mean = optimize_embddings(right)

        sim = cosine_similarity(left_mean, right_mean)[0][0]
        scores.append(sim)
        indices.append(i)

    threshold = np.mean(scores) - 1.2 * np.std(scores)

    boundaries = []
    last = 0

    for idx, score in zip(indices, scores):
        if score < threshold and idx - last >= min_gap:
            boundaries.append(idx)
            last = idx

    return boundaries, scores, indices


def optimize_embddings(left):
    return np.mean(left, axis=0).reshape(1, -1)



from sentence_transformers import SentenceTransformer
from wdr_article_semantic_chunking_2.util.util import generate_combinations, get_path_for_plotting, get_path_for_csv, \
    get_path_for_plotting2

from wdr_article_semantic_chunking_2.computer.plotter import plot_sliding_window
from wdr_article_semantic_chunking_2.computer.slid_win import compute, segment_topics_window
from wdr_article_semantic_chunking_2.text_util.sw_writer import save_pair_to_csv
from wdr_article_semantic_chunking_2.text_util.text_extractor import extract_texts_and_write_to_file


def find_boundaries(
        model_name="paraphrase-multilingual-mpnet-base-v2",
        window_size=3,
        min_gap=3,
        sentences: list = None,
        debug=False,
        expected_boundaries: list = None,
        result_file_name=None
):
    assert sentences is not None
    model = SentenceTransformer(model_name)

    boundaries, scores, indices = segment_topics_window(sentences, window_size=window_size, min_gap=min_gap,
                                                        model=model)
    if debug:
        plot_sliding_window(
            indices=indices,
            scores=scores,
            expected_boundaries=expected_boundaries,
            file_name=get_path_for_plotting2(result_file_name),
            window_size=window_size
        )

        save_pair_to_csv(
            key=result_file_name,
            possible_breaks=boundaries,
            expected_boundary=expected_boundaries,
            source_count=None,
            file_path=get_path_for_csv(result_file_name)
        )
        return (
            boundaries,
            scores,
            indices,
        )
    else:
        return boundaries


if __name__ == "__main__":
    models = [
        "all-MiniLM-L12-v2",
        # "all-MiniLM-L6-v2",
        # "all-distilroberta-v1",
        # "all-mpnet-base-v2",
        # "distiluse-base-multilingual-cased-v1",
        # "distiluse-base-multilingual-cased-v2",
        # "multi-qa-MiniLM-L6-cos-v1",
        # "multi-qa-distilbert-cos-v1",
        # "multi-qa-mpnet-base-dot-v1",
        # "paraphrase-MiniLM-L3-v2",
        # "paraphrase-albert-small-v2",
        # "paraphrase-multilingual-MiniLM-L12-v2",
        # "paraphrase-multilingual-mpnet-base-v2",
    ]
    file_name = f"merged_filtered_{1}.json"
    blocks, expected_boundary, source_count, _ = extract_texts_and_write_to_file(file_name, False)

    # for model in models:
    #     for window_and_gap_comb in generate_combinations():# we generate different combination ( in total 36) of window_size and min_gap
    r = find_boundaries(sentences=blocks, expected_boundaries=expected_boundary, result_file_name=file_name)
    print(r)



import os
import json
import re
from pathlib import Path
import tkinter as tk
from tkinter import filedialog

def extract_sophora_text(data):
    results = []
    if isinstance(data, list):
        for item in data:
            paragraphs = item.get("sophora-extension:paragraph", [])
            for para in paragraphs:
                text = para.get("sophora-extension:text")
                if text:
                    results.append(clean_html(text))
    return results

# Keys to retain in the filtered JSON
REQUIRED_KEYS = [
    "sophora:tags",
    "wdr:leadin",
    "wdr:schlagzeile",
    "wdr:teasertext",
]

# Clean HTML tags (basic)
def clean_html(value):
    if not isinstance(value, str):
        return value
    # Remove specific tags
    value = re.sub(r'<[^>]+>', '', value)
    return value.strip()


# Merge and filter two JSON objects
def merge_and_filter(json1, json2, filename1, filename2):
    merged = {"source_files": [os.path.basename(filename1), os.path.basename(filename2)]}

    value1_fields = {}
    value2_fields = {}

    for key in REQUIRED_KEYS:
        value1 = clean_html(json1.get(key))
        value2 = clean_html(json2.get(key))
        value1_fields[key] = value1
        value2_fields[key] = value2

    sophora_text_1 = clean_html(extract_sophora_text(json1.get("wdr:text", [])))
    sophora_text_2 = clean_html(extract_sophora_text(json2.get("wdr:text", [])))
    value1_fields["sophora-extension:text"] = sophora_text_1
    value2_fields["sophora-extension:text"] = sophora_text_2

    merged["file_contents"] = [
        dict(value1_fields), dict(value2_fields)
    ]

    return merged

# Read all JSON files in the directory
def load_json_files(folder_path):
    files = sorted([f for f in os.listdir(folder_path) if f.endswith('.json')])
    return [os.path.join(folder_path, f) for f in files]


def select_input_folder():
    root = tk.Tk()
    root.withdraw()  # Hide main window
    folder_selected = filedialog.askdirectory(title="Select Folder Containing JSON Files")
    return folder_selected


def main():
    input_folder = select_input_folder()
    if not input_folder:
        print("No folder selected. Exiting.")
        return

    parent_folder = os.path.dirname(input_folder)
    output_folder = os.path.join(parent_folder, "filtered_output")

    Path(output_folder).mkdir(parents=True, exist_ok=True)

    json_paths = load_json_files(input_folder)

    if len(json_paths) < 2:
        print("Not enough JSON files to process.")
        return

    files_buffer = []
    paths_buffer = []
    file: int = 0
    skipped: int = 0
    for i in range(0, len(json_paths), 1):
        try:
            with (open(json_paths[i], 'r', encoding='utf-8') as f1):
                json_i = json.load(f1)
                sophora_text = clean_html(extract_sophora_text(json_i.get("wdr:text", [])))

                if len(sophora_text) < 4:
                    skipped += 1
                    print(f"This file has fewer than 4 texts: {os.path.basename(json_paths[i])}")
                    continue
                else:
                    files_buffer.append(json_i)
                    paths_buffer.append(json_paths[i])

                    if len(files_buffer) == 2:
                        file += 1
                        merged_filtered = merge_and_filter(files_buffer[0], files_buffer[1], paths_buffer[0], paths_buffer[1])
                        output_filename = f'merged_filtered_{file}.json'
                        output_path = os.path.join(output_folder, output_filename)

                        with open(output_path, 'w', encoding='utf-8') as outfile:
                            json.dump(merged_filtered, outfile, ensure_ascii=False, indent=4)

                        files_buffer.clear()
                        paths_buffer.clear()

        except json.JSONDecodeError:
            print(f"Error decoding JSON files: {json_paths[i]}")
            continue

    print(f"\n✅ Processed {len(json_paths)} files into {file} merged JSON files. (Skipped {skipped} files).")
    print(f"📁 Output saved in: {output_folder}")


if __name__ == "__main__":
    main()




from typing import Tuple, List, Any
import json
import logging
from wdr_article_semantic_chunking_2.text_util.sw_writer import write_to_file, write_to_file_with_index, write_split
def read_json(file_path):
    try:
        with open(file_path, 'r', encoding='utf-8') as f:
            return json.load(f)
    except (FileNotFoundError, json.JSONDecodeError) as e:
        logging.error(f"Error reading {file_path}: {e}")
        return {}

# it also marks sentences, segregate text and writes to file for debug
def extract_texts_and_write_to_file(file_name, debug_mode :bool = True)-> tuple[list[Any], list[Any], Any, dict[str,int]]:
    data = read_json(f"/home/nurseiit/PycharmProjects/wdr-article-semantic-chunking-2/data/{file_name}")

    texts = []
    boundaries = []
    current_index = 0
    debug_index = 0
    dict_for_debug = {}
    file_name = f"{file_name}.txt"

    clear_file_content(file_name, debug_mode=debug_mode)

    for index, content in enumerate(data.get("file_contents", [])):
        article_texts = content.get("sophora-extension:text", [])
        for i in article_texts:
            dict_for_debug[debug_index]=i
            write_to_file_with_index(i, line_number=debug_index, output_filename = file_name, should_write=debug_mode)
            debug_index= debug_index + 1
        write_split(output_filename=file_name,should_write=debug_mode)
        texts.extend(article_texts)
        current_index += len(article_texts)
        boundaries.append(current_index - 1)
    source_count = data.get("source_files", 0)
    return texts, boundaries,len(source_count), dict_for_debug


def clear_file_content(file_name: str, debug_mode:bool):
    if debug_mode:
        from pathlib import Path
        folder_name = "content"
        path = Path.cwd() / folder_name / file_name
        path.parent.mkdir(parents=True, exist_ok=True)
        # Open in write mode to clear the file
        with open(path, "w") as f:
            f.write("")  # optional, just opening in 'w' clears it

# file_name = f"merged_filtered_{3}.json"
# original_data, expected_boundary, source_count, er = extract_texts_and_write_to_file(file_name)



import os
from pathlib import Path
import matplotlib.pyplot as plt
import numpy as np


def plot_sliding_window(
        scores,
        indices,
        file_name,
        window_size,
        expected_boundaries=None
):
    plt.figure(figsize=(14, 5))

    plt.bar(indices, scores, label="Cosine similarity")

    threshold = np.mean(scores) - 1.2 * np.std(scores)
    plt.axhline(
        threshold,
        color="red",
        linestyle="--",
        label="Threshold"
    )


    for i, s in zip(indices, scores):
        if s < threshold:
            plt.scatter(i, s, color="red", zorder=3)


    if expected_boundaries:
        for b in expected_boundaries:
            plt.axvline(
                b,
                color="green",
                linestyle=":",
                alpha=0.7,
                label="Expected boundary" if b == expected_boundaries[0] else None
            )

    ax = plt.gca()  # get current axis
    import matplotlib.ticker as mticker
    ax.xaxis.set_major_locator(mticker.MultipleLocator(1))

    plt.xlabel("index of block (between i-1 and i)")
    plt.ylabel("Semantic similarity")
    plt.title(f"Sliding Window Topic Segmentation with window size {window_size}")
    plt.legend()
    plt.tight_layout()

    p = Path(file_name)
    if p.parent:  # only if there's a directory component
        p.parent.mkdir(parents=True, exist_ok=True)

    plt.savefig(file_name)
    plt.close()



import pandas as pd

from wdr_article_semantic_chunking_2.text_util.sw_writer import save_pair_to_csv, MATCH_PERCENTAGE, save_result_tocsv
from llama_index.core.node_parser import SemanticSplitterNodeParser, SemanticDoubleMergingSplitterNodeParser, \
    LanguageConfig
from llama_index.embeddings.huggingface import HuggingFaceEmbedding
from wdr_article_semantic_chunking_2.text_util.text_extractor import extract_texts_and_write_to_file
from llama_index.core import Document, Settings

from wdr_article_semantic_chunking_2.util.util import get_path_for_csv


def compute(
        model_name,
        initial_threshold=0.45,
        appending_threshold=0.35,
        merging_threshold=0.35,
        max_chunk_size=1200,
        merging_range=2,
):
    Settings.embed_model = HuggingFaceEmbedding(
        model_name=model_name
    )
    combination_name = f"model_{model_name}_initTresh_{initial_threshold}_apTresh_{appending_threshold}_mTresh_{merging_threshold}_maxChunk_{max_chunk_size}_mRange_{merging_range}"
    for i in range(0, 100):
        file_name = f"merged_filtered_{i}.json"
        print(file_name)
        blocks, expected_boundary, source_count, debug_dict = extract_texts_and_write_to_file(file_name, False)

        splitter = SemanticDoubleMergingSplitterNodeParser(
            language_config=LanguageConfig(language="german", spacy_model="de_core_news_md"),
            sentence_splitter=lambda text: blocks,

            # ---- thresholds ----
            initial_threshold=initial_threshold,
            appending_threshold=appending_threshold,
            merging_threshold=merging_threshold,

            # ---- merging ----
            max_chunk_size=max_chunk_size,
            merging_range=merging_range,
            merging_separator=" ",
        )
        doc = Document(text=" ".join(blocks))
        nodes = splitter.get_nodes_from_documents([doc])
        inverted = {v: k for k, v in debug_dict.items()}
        boundaries = get_approximate_boundary(nodes, debug_dict)

        save_pair_to_csv(
            key=file_name,
            possible_breaks=boundaries,
            expected_boundary=expected_boundary,
            source_count=source_count,
            file_path=get_path_for_csv(combination_name)
        )
    df = pd.read_csv(get_path_for_csv(combination_name), usecols=[MATCH_PERCENTAGE])
    save_result_tocsv(combination_name, df.mean().iloc[0])


def get_approximate_boundary(nodes, dict_for_debug):
    print(f"Total Chunks: {len(nodes)}")
    aproximate_index = []
    for i, node in enumerate(nodes, start=0):
        for t in range(0, max(dict_for_debug.keys())):
            debug = dict_for_debug[t].split()[:10]
            debug_sub1 = dict_for_debug.get(t - 1, "").split()[:10]
            debug_sub2 = dict_for_debug.get(t - 2, "").split()[:10]
            debug_add1 = dict_for_debug.get(t + 1, "").split()[:10]
            debug_add2 = dict_for_debug.get(t + 2, "").split()[:10]

            if len(debug) > 0 and " ".join(node.get_content().split()[:len(debug)]) == " ".join((debug)):
                aproximate_index.append(t)
                break
            if len(debug_sub1) > 0 and " ".join(node.get_content().split()[:len(debug_sub1)]) == " ".join((debug_sub1)):
                aproximate_index.append(t - 1)
                break
            if len(debug_sub2) > 0 and " ".join(node.get_content().split()[:len(debug_sub2)]) == " ".join((debug_sub2)):
                aproximate_index.append(t - 2)
                break

            if len(debug_add1) > 0 and " ".join(node.get_content().split()[:len(debug_add1)]) == " ".join((debug_add1)):
                aproximate_index.append(t + 1)
                break
            if len(debug_add2) > 0 and " ".join(node.get_content().split()[:len(debug_add2)]) == " ".join((debug_add2)):
                aproximate_index.append(t + 2)
                break
    return aproximate_index


if __name__ == "__main__":
    models = [
        # "all-MiniLM-L12-v2",
        # "all-MiniLM-L6-v2",
        # "all-distilroberta-v1",
        # "all-mpnet-base-v2",
        # "distiluse-base-multilingual-cased-v1",
        # "distiluse-base-multilingual-cased-v2",
        # "multi-qa-MiniLM-L6-cos-v1",
        # "multi-qa-distilbert-cos-v1",
        # "multi-qa-mpnet-base-dot-v1",
        # "paraphrase-MiniLM-L3-v2",
        # "paraphrase-albert-small-v2",
        "paraphrase-multilingual-MiniLM-L12-v2",
        # "paraphrase-multilingual-mpnet-base-v2",
    ]
    for model in models:
        # combination_name = f"model_{model}_buf_{0}_t_{80}"
        # df = pd.read_csv(get_path_for_csv(combination_name), usecols=[MATCH_PERCENTAGE])
        # save_result_tocsv(combination_name, df.mean().iloc[0])
        compute(
            initial_threshold=0.9,
            appending_threshold=0.20,
            merging_threshold=0.40,
            max_chunk_size=1500,
            merging_range=2,
            model_name=model)



from wdr_article_semantic_chunking_2.text_util.sw_writer import match_with_tolerance, match_with_tolerance2


import pytest

@pytest.mark.parametrize(
    "expected, possible, expected_output",
    [
        ([10, 20, 30], [10, 20, 30], {0: True, 1: True}),
        ([10, 20, 30], [12, 23], {0: True, 1: True}),
        ([5, 15], [],  {0: False, }),
        ([9, 21, 34, 57], [9, 10, 22, 23, 26, 35, 36, 47],  {0:True,1:True,2:True, }),
        ([11, 26, 43, 54], [0, 5, 11, 14, 20, 34, 37, 41, 46, 51, 53, 54],  {0:True,1:False,2:True, }),

    ]
)
def test_match_with_tolerance2(expected, possible, expected_output):
    assert match_with_tolerance2(expected, possible) == expected_output



import os



def get_path_for_csv(combination_name):
    PATH = f"result/{combination_name}/"
    FILE_NAME = f"{combination_name}.csv"
    file_path = f"{PATH}{FILE_NAME}"
    return file_path

def get_path_for_plotting(combination_name:str,file_name: str) -> str:
    file_without_ext = os.path.splitext(file_name)[0]
    output_path = f"result/{combination_name}/{file_without_ext}/{file_name}.png"
    return output_path

def get_path_for_plotting2(file_name: str) -> str:
    file_without_ext = os.path.splitext(file_name)[0]
    output_path = f"result/{file_without_ext}/{file_name}.png"
    return output_path

from itertools import product

def generate_combinations(window_range=range(1, 6), gap_range=range(1, 6)):
    return [{"window_size": w, "min_gap": g} for w, g in product(window_range, gap_range)]